package jalview.gui;

import jalview.jbgui.GColourKey;
import jalview.schemes.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;


public class ColourKey extends GColourKey
{

 Color midBlue = new Color(100,100,255);
 public ColourKey(ColourSchemeI current)
 {
   if(current==null)
        zappoMenuItem_actionPerformed(null);

   else if(current instanceof Blosum62ColourScheme)
        blosumMenuItem_actionPerformed(null);

  else if(current instanceof BuriedColourScheme)
    buriedMenuItem_actionPerformed(null);

//  else if(current instanceof ClustalxColourScheme)
 //   cl
  else if(current instanceof HelixColourScheme)
    helixMenuItem_actionPerformed(null);

  else if(current instanceof HydrophobicColourScheme)
    hydroMenuItem_actionPerformed(null);

  else if(current instanceof StrandColourScheme)
    strandMenuItem_actionPerformed(null);

  else if(current instanceof TaylorColourScheme)
    taylorMenuItem_actionPerformed(null);

  else if(current instanceof TurnColourScheme)
    turnMenuItem_actionPerformed(null);

  else // default
     zappoMenuItem_actionPerformed(null);
 }

 void displayKey(String [] residue, String [] description, Color []  colour, String comment)
 {
   this.getContentPane().removeAll();

   for(int i=0; i<colour.length; i++)
   {
     JPanel rowPanel = new JPanel();
     JLabel residueLabel = new JLabel();
     JLabel textLabel = new JLabel();
     JPanel colourPanel = new JPanel();
     rowPanel.setLayout(new BorderLayout());
     residueLabel.setFont(new java.awt.Font("Verdana", Font.BOLD, 12));
     residueLabel.setPreferredSize(new Dimension(80, 20));
     if(residue!=null)
       residueLabel.setText(residue[i]);
     residueLabel.setOpaque(false);
     residueLabel.setHorizontalAlignment(SwingConstants.CENTER);
     residueLabel.setHorizontalTextPosition(SwingConstants.CENTER);
     textLabel.setFont(new java.awt.Font("Verdana", 0, 12));
     textLabel.setPreferredSize(new Dimension(250,20));
     textLabel.setBackground(Color.white);
     if(description != null)
        textLabel.setText(description[i]);

     colourPanel.setBackground(colour[i]);
     colourPanel.setBorder(BorderFactory.createRaisedBevelBorder());
     colourPanel.setPreferredSize(new Dimension(80, 20));
     colourPanel.setLayout(new BorderLayout());
     colourPanel.add(residueLabel, BorderLayout.CENTER);

     rowPanel.add(colourPanel, BorderLayout.CENTER);
     if(description!=null)
       rowPanel.add(textLabel, BorderLayout.WEST);
     this.getContentPane().add(rowPanel);
   }

   if(comment!=null)
   {
     JTextArea ta = new JTextArea(comment);
     ta.setLineWrap(true);
     ta.setWrapStyleWord(true);
     ta.setPreferredSize( new Dimension(this.getWidth()-20, 120) );
     this.getContentPane().add(ta);
   }
   validate();
   repaint();
 }


protected void clustalMenuItem_actionPerformed(ActionEvent e)
{
  menu.setText("ClustalX colour scheme");
  this.getContentPane().removeAll();
  repaint();
}

protected void zappoMenuItem_actionPerformed(ActionEvent e)
{
    menu.setText("Zappo colour scheme");
    displayKey(
        new String[]{"ILVAM", "FWY", "KRH", "DE", "STNQ", "PG", "C"},
        new String[]{"Aliphatic/hydrophobic residues", "Aromatic",
        "Positive", "Negative", "Hydrophilic", "conformationally special", "Cysteine"},
        new Color[]{ Color.pink, Color.orange, midBlue, Color.red, Color.green, Color.magenta, Color.yellow}
        ,null
        );

}

protected void taylorMenuItem_actionPerformed(ActionEvent e)
{
  menu.setText("Taylor colour scheme");
  String [] res = new String[]{"A","V","I","L","M","F","Y","W","H","R","K","N","Q","E","D","S","T","G","P","C"};
  Color [] col = new Color[res.length];
  for (int i=0; i<res.length; i++)
    col[i] = ResidueProperties.taylor[ ((Integer)ResidueProperties.aaHash.get(res[i])).intValue() ];


  displayKey( res, null, col, "These colours were invented by Willie Taylor and an entertaining description of their birth can be found in Protein Engineering, Vol 10 , 743-746 (1997)"
 );
}

protected void hydroMenuItem_actionPerformed(ActionEvent e)
{

  menu.setText("Hydrophobicity colour scheme");
  String [] res = new String[]{"I","V","L","F","C","M","A","G","X","T","S","W","Y","P","H","E","Z","Q","D","B","N","K","R"};
  Color [] col = new Color[res.length];
  HydrophobicColourScheme cs = new HydrophobicColourScheme();
  for (int i=0; i<res.length; i++)
    col[i] = cs.findColour( res[i], -1,  null);

  displayKey( res, null, col, "According to the hydrophobicity table of  Kyte, J., and Doolittle, R.F., J. Mol. Biol. 1157, 105-132, 1982.  The most hydrophobic residues according to this table are coloured red and the most hydrophilic ones are coloured blue.");

}

protected void helixMenuItem_actionPerformed(ActionEvent e)
{
  menu.setText("Helix propensity colour scheme");
  String[] res = new String[]
      {"E","M","A","Z","L","K","F","Q","I","W","V","D","X","H","R","B","T","S","C","Y","N","G","P"};
  Color[] col = new Color[res.length];
  HelixColourScheme cs = new HelixColourScheme();
  for (int i = 0; i < res.length; i++)
    col[i] = cs.findColour( res[i], -1, null);

  displayKey(res, null, col, null);
}

protected void strandMenuItem_actionPerformed(ActionEvent e)
{
  menu.setText("Strand propensity colour scheme");
  String[] res = new String[]
    {"V","I","Y","F","W","L","T","C","Q","M","X","R","N","H","A","S","G","Z","K","B","P","D","E"};
Color[] col = new Color[res.length];
StrandColourScheme cs = new StrandColourScheme();
for (int i = 0; i < res.length; i++)
  col[i] = cs.findColour( res[i], -1, null);
  displayKey(res, null, col, null);
}

protected void turnMenuItem_actionPerformed(ActionEvent e)
{
  menu.setText("Turn propensity colour scheme");
  String[] res = new String[]
    {"N","G","P","B","D","S","C","Y","K","X","Q","W","T","R","H","Z","E","A","F","M","L","V","I"};
Color[] col = new Color[res.length];
TurnColourScheme cs = new TurnColourScheme();
for (int i = 0; i < res.length; i++)
  col[i] = cs.findColour( res[i], -1, null);

  displayKey(res, null, col, null);
}

protected void buriedMenuItem_actionPerformed(ActionEvent e)
{
  menu.setText("Buried Index colour scheme");
  String[] res = new String[]
    {"C","I","V","L","F","M","G","A","W","X","S","H","T","P","Y","N","B","D","Q","Z","E","R","K"};
Color[] col = new Color[res.length];
BuriedColourScheme cs = new BuriedColourScheme();
for (int i = 0; i < res.length; i++)
  col[i] = cs.findColour( res[i], -1, null);

  displayKey(res, null, col, null);
}

protected void blosumMenuItem_actionPerformed(ActionEvent e)
{
  menu.setText("BLOSUM62 colour scheme");
  displayKey( null,
              new String[]{"Consensus","+ve Blosum62", " "},
              new Color[]{ new Color(154, 154, 255), new Color(204, 204, 255),  Color.white },
              "Gaps are coloured white. If a residue matchs the consensus sequence residue at that position"
             +" it is colored dark blue. If it does not match the consensus residue but the 2 residues have a positive Blosum62 score, it is colored light blue.");

}
}


/*
Blosum62 - Sequence positions which are gaps are colored white. For sequence positions which are non-gaps,
if a residue matchs the consensus sequence residue at that position, it is colored dark blue.
If it does not match the consensus residue but the 2 residues have a positive Blosum62 score, it is colored light blue.
Otherwise, it is colored white.
Percentage Identity - Sequence positions which are gaps are colored white. For sequence positions which are non-gaps,
if the residue at a given position is conserved in less than 40% of the sequences in the alignment, it is colored white.
For residues conserved in greater than 40% of the sequences in the alignment, a shade of blue is assigned depending on
the level of conservation, with darker blues corresponding to residues that are more conserver.
*/

   /*
   This depends on the applet having performed a consensus calculation on the alignment.  This is done automatically when the applet first starts up but is not done for subsequent alignment windows.  If the consensus seuqence is calculated again by selecting 'Consensus' from the Calculate menu then the consensus is calculated for the selected sequences only  and all the alignment sequences are coloured according to that consensus.  This can be useful to display how similar all the other sequences in the alignment are compared to one (or a few) selected sequences.
The PID option colours the residues (boxes and/or text) according to the percentage of the residues in each column that agree with the consensus sequence.  Only the residues that agree with the consensus residue for each column are coloured.

  Percentage agreement Colour
> 80 % Mid blue
> 60 % Light blue
> 40 % Light grey
<= 40% White
*/














